function [lik, all_s_up, Neff, len_phis] = tempered_PF_lik(A, B, H, Phi, R, S2, N, yt, x0, P0, tune)
% -----------------------------------------------------------------------
% tempered particle filtering for DSGE model with measurement error
%
% Author: Minsu Chang        minsuc@sas.upenn.edu
% Last modified: 5/8/2017
% -----------------------------------------------------------------------
%                        Model( Linear Model)
%               y(t) = A + B*s(t) + u(t), u(t) ~ N(0,H)
%           s(t) = Phi*s(t-1) + R*e(t), e(t) ~ iid N(0,S2)
% -----------------------------------------------------------------------
% Input
%   A, B, H : parameters for measurement equation
%   Phi, R, S2 : parameters for transition equation
%   N       : number of particles
%   yt      : data
%   x0, P0  : initial prior for the filter
% -----------------------------------------------------------------------
% Output
%   lik       : likelihood (T by 1)
%   all_s_up  : particles resampled (updated) (T by N)
%   Neff      : Effective sample size
% -----------------------------------------------------------------------

% housekeeping
ne        = size(S2,1);
[~, ns]   = size(B);
T         = size(yt,1);
sqrtS2    = R*chol(S2)';

% matrix for store
all_s_up  = zeros(T, ns, N);   % resampled
lik       = zeros(T,1);
Neff      = zeros(T,1);
len_phis  = ones(T,1);

% initialization
temp_s = x0;
temp_P = P0;
s_up   = repmat(temp_s, 1, N) + chol(temp_P)'*randn(ns, N);

weights = ones(N,1);


for tt=1:1:T

        fprintf('-----------------------------------------------\n')
        fprintf(' Time Period = %d    \n', tt);        

        yy = yt(tt,:);
    
        % Propagate       
        eps     = randn(ne, N);
        s_fore  = Phi*s_up + sqrtS2*eps;
        
        % Un-normalized weights (naive boostrap particle filter)
        perror  = repmat(yy'-A, 1, N) - B*s_fore;
        
        % initial tempering phi_1
        init_InEff_func = @(phi)(init_InEff(phi,perror,H)-tune.rstar);
        phi_interval = [0.000001 1.0]; % initial interval (note that the signs at the end points should differ)
        phi_1 = fzero(init_InEff_func, phi_interval);
        %phi_1 = 1; % to obtain BS PF;
        
        fprintf(' Initial phi  = %2.5f    \n', phi_1);
        
        density = zeros(N,1);   
            for nn = 1:N
                density(nn,1) = (2*pi)^(-size(H,1)/2)*phi_1^(size(H,1)/2)*det(H)^(-1/2)* ...
                    exp(-1/2*perror(:,nn)'*phi_1*inv(H)*perror(:,nn));
            end
            
        lik(tt,1)  = log(mean(density.*weights));    
            
        weights = (density.*weights)./(mean(density.*weights)); 
      
        % Resampling 
        % we keep track of s(t), s(t-1), and eps(t);
        id_up   = randsample(N,N,true,weights)';
        s_up    = s_up(:,id_up);  
        eps_up  = eps(:,id_up); 
        s_fore  = s_fore(:,id_up);
        weights = ones(N,1);

        
        % tempering iterations
        count   = 1;
        phi_old = phi_1;
                
        while (phi_old < 1)
            
            count         = count + 1;
            
            fprintf(' Temp Iteration n = %2.5f    \n', count);
            
            % compute forecast error
            perror        = repmat(yy'-A, 1, N) - B*s_fore;
            
            % determine desired phi
            InEff_func    = @(phi)(InEff(phi, phi_old, perror, H) - tune.rstar);
            phi_interval  = [phi_old 1.0];
            fphi_interval = [InEff_func(phi_old) InEff_func(1.0)];
            if prod(sign(fphi_interval))==-1
               % it is optimal to set phi_new < 1 
                phi_new = fzero(InEff_func, phi_interval);
            else
                phi_new = 1;
            end
            
            fprintf(' phi = %2.5f    \n', phi_new);
            
            % correction: compute unnormalized weights
            for nn = 1:N
                   density(nn,1) = (phi_new/phi_old)^(size(H,1)/2)* ...
                   exp(-1/2*perror(:,nn)'*(phi_new-phi_old)*inv(H)*perror(:,nn));
            end
            % correction: compute likelihood increment
            lik(tt,1)  = lik(tt,1)+log(mean(density.*weights));
            % correction: compute normalized weights
            weights = (density.*weights)./(mean(density.*weights));   
            
            fprintf(' log like(n,t) = %2.5f    \n', lik(tt,1));
          
            % resampling 
            % we keep track of s(t), s(t-1), and eps(t);
            id_up   = randsample(N,N,true,weights)';
            s_up    = s_up(:,id_up); 
            eps_up  = eps_up(:,id_up);
            s_fore  = s_fore(:,id_up);
            weights = ones(N,1) ;
            
            % tune the mutation step
            tune.c  = tune.c*(0.95 + 0.10*exp(20*(tune.acpt-tune.trgt))/(1 + ...
                     exp(20*(tune.acpt-tune.trgt))));
            tune.mu = mean(eps_up,2);
            tune.R  = eye(ne);
           
            % mutation
            temp_acpt = zeros(N,1); %initialize accpetance indicator
            parfor nn = 1:N
               [s_fore_new, eps_new, ind_acpt]  = mutation_RWMH_multistep(s_fore(:,nn), ...
                   s_up(:,nn), eps_up(:,nn), Phi, sqrtS2, A, B, H/phi_new, yy, tune)
               eps_up(:,nn)    = eps_new; 
               s_fore(:,nn)    = s_fore_new;
               temp_acpt(nn,1) = ind_acpt;
            end 
            tune.acpt = mean(temp_acpt); % update average acceptance rate
     
            phi_old = phi_new;
            len_phis(tt) = len_phis(tt)+1;
     

        end
       
        % Store results
        Neff(tt,1) = (N^2)/sum(weights.^2);
        s_up = s_fore;
        all_s_up(tt,:,:) = s_up;               
      
end



